@@ -52,10 +52,11 @@ var shellCmd = &cobra.Command{
 
 			err := handler.RunAll(ctx, args)
 
-			// Don't bother printing the error message if the TUI is enabled.
+			// Wrap exit status in ExitError so the TUI preserves the exit code
+			// and doesn't print a redundant error message.
 			var es interp.ExitStatus
 			if handler.tty && errors.As(err, &es) {
-				return idtui.ExitError{Code: int(es)}
+				return idtui.ExitError{Code: int(es), Original: err}
 			}
 
 			return err
