@@ -8,6 +8,7 @@ import (
 	"time"
 
 	"go.opentelemetry.io/otel/attribute"
+	"go.opentelemetry.io/otel/codes"
 	sdklog "go.opentelemetry.io/otel/sdk/log"
 	sdkmetric "go.opentelemetry.io/otel/sdk/metric"
 	"go.opentelemetry.io/otel/sdk/metric/metricdata"
@@ -369,6 +370,7 @@ func (db *DB) newSpan(spanID SpanID) *Span {
 		RevealedSpans:   NewSpanSet(),
 		FailedLinks:     NewSpanSet(),
 		CanceledLinks:   NewSpanSet(),
+		ErrorOrigins:    NewSpanSet(),
 		causesViaLinks:  NewSpanSet(),
 		effectsViaLinks: NewSpanSet(),
 		db:              db,
@@ -647,10 +649,15 @@ func (db *DB) integrateSpan(span *Span) { //nolint: gocyclo
 				continue
 			}
 			linked := db.initSpan(linkedCtx.SpanID)
-			span.ErrorOrigin = linked
-			if linked.HasParent(span) {
-				span.RevealedSpans.Add(linked)
-			}
+			span.ErrorOrigins.Add(linked)
+		}
+	}
+
+	// Extract error origins from span error descriptions
+	if span.Status.Code == codes.Error {
+		for _, origin := range telemetry.ParseErrorOrigins(span.Status.Description) {
+			linked := db.initSpan(SpanID{SpanID: origin.SpanID()})
+			span.ErrorOrigins.Add(linked)
 		}
 	}
 
@@ -668,7 +675,9 @@ func (db *DB) integrateSpan(span *Span) { //nolint: gocyclo
 
 	if !span.ParentID.IsValid() && span.Received {
 		// keep track of the trace's root span
-		db.RootSpan = span
+		if db.RootSpan == nil {
+			db.RootSpan = span
+		}
 
 		if !db.PrimarySpan.IsValid() {
 			// default primary to root span, though we might never see a "root
@@ -676,10 +685,13 @@ func (db *DB) integrateSpan(span *Span) { //nolint: gocyclo
 			db.PrimarySpan = span.ID
 		}
 
-		if !span.IsRunning() {
+		if span == db.RootSpan && !span.IsRunning() {
+			// If the root span is completed, we should mark any still-running spans
+			// as canceled.
 			for _, span := range db.Spans.Order {
 				if span.IsRunning() {
 					span.Canceled = true
+					span.LeftRunning = true
 					span.EndTime = db.RootSpan.EndTime
 					span.PropagateStatusToParentsAndLinks()
 					db.update(span)
@@ -687,7 +699,10 @@ func (db *DB) integrateSpan(span *Span) { //nolint: gocyclo
 			}
 		}
 	} else if db.RootSpan != nil && !db.RootSpan.IsRunning() && span.IsRunning() {
+		// Same as above (cancel running spans when root ends), but handled for
+		// incoming span updates too.
 		span.Canceled = true
+		span.LeftRunning = true
 		span.EndTime = db.RootSpan.EndTime
 		span.PropagateStatusToParentsAndLinks()
 	}
@@ -896,19 +911,6 @@ func (db *DB) Simplify(call *callpbv1.Call, force bool) *callpbv1.Call {
 	return call
 }
 
-// Function to check if a row is or contains a target row
-func isOrContains(row, target *TraceTree) bool {
-	if row == target {
-		return true
-	}
-	for _, child := range row.Children {
-		if isOrContains(child, target) {
-			return true
-		}
-	}
-	return false
-}
-
 type WalkDecision int
 
 const (
@@ -937,34 +939,6 @@ func WalkTree(tree []*TraceTree, f func(*TraceTree, int) WalkDecision) {
 	walk(tree, 0)
 }
 
-func (db *DB) CollectErrors(rows *RowsView) []*TraceTree {
-	reveal := make(map[*TraceTree]struct{})
-
-	var collect func(row *TraceTree, ancestorFailed bool)
-	collect = func(tree *TraceTree, ancestorFailed bool) {
-		failed := tree.Span.IsFailedOrCausedFailure()
-		unset := tree.Span.IsUnset()
-		if failed {
-			reveal[tree] = struct{}{}
-		}
-		if failed ||
-			// continue through UNSET spans beneath failed parents; these correspond
-			// to basic span.End() calls which intend to just add measurement without
-			// bubbling up or masking failure
-			(unset && ancestorFailed) {
-			for _, child := range tree.Children {
-				collect(child, true)
-			}
-		}
-	}
-
-	for _, row := range rows.Body {
-		collect(row, false)
-	}
-
-	return collectParents(rows.Body, reveal)
-}
-
 func (db *DB) FindResource(filter attribute.KeyValue) *resource.Resource {
 	for _, res := range db.Resources {
 		for _, kv := range res.Attributes() {
@@ -975,25 +949,3 @@ func (db *DB) FindResource(filter attribute.KeyValue) *resource.Resource {
 	}
 	return nil
 }
-
-func collectParents(rows []*TraceTree, targets map[*TraceTree]struct{}) []*TraceTree {
-	var result []*TraceTree
-
-	for _, row := range rows {
-		contains := false
-		for target := range targets {
-			if isOrContains(row, target) {
-				contains = true
-				break
-			}
-		}
-		if !contains {
-			continue
-		}
-		rowCopy := *row
-		rowCopy.Children = collectParents(row.Children, targets)
-		result = append(result, &rowCopy)
-	}
-
-	return result
-}
