@@ -17,6 +17,74 @@ import (
 
 type SpanSet = *OrderedSet[SpanID, *Span]
 
+// spanStateCategory represents the primary state of a span for rollup counting
+type spanStateCategory uint8
+
+const (
+	stateUnknown spanStateCategory = iota
+	stateRunning
+	statePending
+	stateCached
+	stateCanceled
+	stateFailed
+	stateSuccess
+)
+
+// RollUpState caches the computed state for rendering RollUp progress bars
+type RollUpState struct {
+	PendingCount  int
+	RunningCount  int
+	CachedCount   int
+	SuccessCount  int
+	FailedCount   int
+	CanceledCount int
+}
+
+func (st *RollUpState) Reset() {
+	st.PendingCount = 0
+	st.RunningCount = 0
+	st.CachedCount = 0
+	st.SuccessCount = 0
+	st.FailedCount = 0
+	st.CanceledCount = 0
+}
+
+// incrementCategory adds 1 to the count for the given category
+func (st *RollUpState) incrementCategory(cat spanStateCategory) {
+	switch cat {
+	case stateRunning:
+		st.RunningCount++
+	case statePending:
+		st.PendingCount++
+	case stateCached:
+		st.CachedCount++
+	case stateCanceled:
+		st.CanceledCount++
+	case stateFailed:
+		st.FailedCount++
+	case stateSuccess:
+		st.SuccessCount++
+	}
+}
+
+// decrementCategory subtracts 1 from the count for the given category
+func (st *RollUpState) decrementCategory(cat spanStateCategory) {
+	switch cat {
+	case stateRunning:
+		st.RunningCount--
+	case statePending:
+		st.PendingCount--
+	case stateCached:
+		st.CachedCount--
+	case stateCanceled:
+		st.CanceledCount--
+	case stateFailed:
+		st.FailedCount--
+	case stateSuccess:
+		st.SuccessCount--
+	}
+}
+
 type Span struct {
 	SpanSnapshot
 
@@ -26,8 +94,7 @@ type Span struct {
 	FailedLinks   SpanSet `json:"-"`
 	CanceledLinks SpanSet `json:"-"`
 	RevealedSpans SpanSet `json:"-"`
-
-	ErrorOrigin *Span `json:"-"`
+	ErrorOrigins  SpanSet `json:"-"`
 
 	callCache *callpbv1.Call
 	baseCache *callpbv1.Call
@@ -43,6 +110,13 @@ type Span struct {
 	// just allocated due to a span parent or other relationship.
 	Received bool
 
+	// Pre-computed RollUp state for rendering progress bars
+	// Maintained incrementally for all spans, not just those marked RollUp
+	rollUpState *RollUpState
+
+	// Cached state classification for incremental updates
+	lastRollUpCategory spanStateCategory
+
 	db *DB
 }
 
@@ -159,13 +233,25 @@ type SpanSnapshot struct {
 	Canceled bool `json:",omitempty"`
 	Cached   bool `json:",omitempty"`
 
+	// An extra flag to indicate that a span was canceled because the root span
+	// completed while the span was still running.
+	LeftRunning bool `json:",omitempty"`
+
 	// UI preferences reported by the span, or applied to it (sync=>passthrough)
 	Internal     bool `json:",omitempty"`
 	Encapsulate  bool `json:",omitempty"`
 	Encapsulated bool `json:",omitempty"`
 	Passthrough  bool `json:",omitempty"`
 	Ignore       bool `json:",omitempty"`
-	Reveal       bool `json:",omitempty"`
+
+	Boundary    bool `json:",omitempty"`
+	Reveal      bool `json:",omitempty"`
+	RollUpLogs  bool `json:",omitempty"`
+	RollUpSpans bool `json:",omitempty"`
+
+	// Check name + status
+	CheckName   string `json:",omitempty"`
+	CheckPassed bool   `json:",omitempty"`
 
 	ActorEmoji  string `json:",omitempty"`
 	Message     string `json:",omitempty"`
@@ -199,7 +285,7 @@ type SpanLink struct {
 	Purpose     string
 }
 
-func (snapshot *SpanSnapshot) ProcessAttribute(name string, val any) {
+func (snapshot *SpanSnapshot) ProcessAttribute(name string, val any) { //nolint: gocyclo
 	defer func() {
 		// a bit of a shortcut, but there shouldn't be much going on
 		// here and all the conversion error handling code is
@@ -234,6 +320,9 @@ func (snapshot *SpanSnapshot) ProcessAttribute(name string, val any) {
 	case telemetry.UIRevealAttr:
 		snapshot.Reveal = val.(bool)
 
+	case telemetry.UIBoundaryAttr:
+		snapshot.Boundary = val.(bool)
+
 	case telemetry.UIInternalAttr:
 		snapshot.Internal = val.(bool)
 
@@ -246,6 +335,19 @@ func (snapshot *SpanSnapshot) ProcessAttribute(name string, val any) {
 	case telemetry.UIMessageAttr:
 		snapshot.Message = val.(string)
 
+	case telemetry.UIRollUpLogsAttr:
+		snapshot.RollUpLogs = val.(bool)
+
+	case telemetry.UIRollUpSpansAttr:
+		snapshot.RollUpSpans = val.(bool)
+
+	case telemetry.CheckNameAttr:
+		snapshot.CheckName = val.(string)
+
+	case telemetry.CheckPassedAttr:
+		// TODO: redundant with span status?
+		snapshot.CheckPassed = val.(bool)
+
 	case telemetry.LLMRoleAttr:
 		snapshot.LLMRole = val.(string)
 
@@ -315,6 +417,9 @@ func sliceOf[T any](val any) []T {
 // NOTE: failed state only propagates to spans that installed the current
 // span's effect - it does _not_ propagate through the parent span.
 func (span *Span) PropagateStatusToParentsAndLinks() {
+	// Update the span's own activity to reflect its current state
+	span.Activity.Add(span)
+
 	propagate := func(parent *Span, causal, activity bool) bool {
 		var changed bool
 		if span.IsRunningOrEffectsRunning() {
@@ -322,9 +427,6 @@ func (span *Span) PropagateStatusToParentsAndLinks() {
 		} else {
 			changed = parent.RunningSpans.Remove(span)
 		}
-		if span.Reveal || parent.ErrorOrigin == span {
-			changed = parent.RevealedSpans.Add(span) || changed
-		}
 		if causal && span.IsFailed() {
 			changed = parent.FailedLinks.Add(span) || changed
 		}
@@ -359,6 +461,153 @@ func (span *Span) PropagateStatusToParentsAndLinks() {
 			}
 		}
 	}
+
+	// Handle revealed spans propagation separately to stop at revealed parents
+	if span.Reveal {
+		for parent := range span.Parents {
+			if parent.RevealedSpans.Add(span) {
+				span.db.update(parent)
+			}
+
+			if parent.Boundary || parent.Encapsulate || parent.Reveal {
+				break
+			}
+		}
+	}
+
+	// Update RollUp state for ancestors incrementally
+	span.updateRollUpAncestors()
+}
+
+// currentStateCategory determines the span's current state category for rollup counting
+func (span *Span) currentStateCategory() spanStateCategory {
+	if span.IsRunningOrEffectsRunning() {
+		return stateRunning
+	} else if span.IsPending() {
+		return statePending
+	} else if span.IsCached() {
+		return stateCached
+	} else if span.IsCanceled() {
+		return stateCanceled
+	} else if span.IsFailedOrCausedFailure() {
+		return stateFailed
+	} else {
+		// Success (completed but not cached, canceled, or failed)
+		return stateSuccess
+	}
+}
+
+// updateRollUpAncestors incrementally updates RollUp state for all ancestors
+// This is O(depth) instead of O(descendants) per update
+func (span *Span) updateRollUpAncestors() {
+	// Determine the span's current state category
+	newCategory := span.currentStateCategory()
+	oldCategory := span.lastRollUpCategory
+
+	// If the category hasn't changed, no update needed
+	if newCategory == oldCategory {
+		return
+	}
+
+	// Update the cached category
+	span.lastRollUpCategory = newCategory
+
+	// Use a map to track which ancestors we've already updated to avoid redundant work
+	updated := make(map[SpanID]bool)
+
+	// Use a queue to iteratively process ancestors instead of recursion
+	var queue []*Span
+
+	// Add immediate parents to queue
+	if span.ParentSpan != nil {
+		queue = append(queue, span.ParentSpan)
+	}
+
+	// Add causal parents to queue
+	span.CausalSpans(func(causal *Span) bool {
+		queue = append(queue, causal)
+		return true
+	})
+
+	// Process ancestors iteratively
+	for len(queue) > 0 {
+		current := queue[0]
+		queue = queue[1:]
+
+		// Skip if already updated
+		if updated[current.ID] {
+			continue
+		}
+		updated[current.ID] = true
+
+		// Lazily initialize rollUpState for all spans
+		if current.rollUpState == nil {
+			current.rollUpState = &RollUpState{}
+		}
+
+		// Incrementally update this ancestor's counts
+		if oldCategory != stateUnknown {
+			current.rollUpState.decrementCategory(oldCategory)
+		}
+		current.rollUpState.incrementCategory(newCategory)
+
+		// Add this ancestor's parents to queue
+		if current.ParentSpan != nil {
+			queue = append(queue, current.ParentSpan)
+		}
+
+		// Add this ancestor's causal parents to queue
+		current.CausalSpans(func(causal *Span) bool {
+			queue = append(queue, causal)
+			return true
+		})
+	}
+}
+
+// initializeRollUpState computes the initial RollUp state by traversing descendants once
+// This can be used to rebuild state from scratch if needed (e.g., for debugging or recovery).
+// Under normal operation, the incremental updates in updateRollUpAncestors are sufficient.
+//
+//nolint:unused
+func (span *Span) initializeRollUpState() {
+	if span.rollUpState == nil {
+		span.rollUpState = &RollUpState{}
+	} else {
+		span.rollUpState.Reset()
+	}
+
+	// Recursively count all descendants
+	for child := range span.Descendants {
+		category := child.currentStateCategory()
+		span.rollUpState.incrementCategory(category)
+		// Also initialize the child's cached category for future incremental updates
+		child.lastRollUpCategory = category
+	}
+}
+
+// Descendants recursively iterates through all descendant spans
+func (span *Span) Descendants(f func(*Span) bool) {
+	var collect func(*Span) bool
+	collect = func(s *Span) bool {
+		// Use ChildSpans directly since we don't have opts here
+		for _, child := range s.ChildSpans.Order {
+			if !f(child) {
+				return false
+			}
+			// Recursively collect grandchildren
+			if !collect(child) {
+				return false
+			}
+		}
+		return true
+	}
+
+	collect(span)
+}
+
+// RollUpState returns the pre-computed RollUp state for rendering progress bars
+func (span *Span) RollUpState() *RollUpState {
+	return span.rollUpState
 }
 
 func (span *Span) ChildOrRevealedSpans(opts FrontendOpts) (SpanSet, bool) {
@@ -450,8 +699,7 @@ func (span *Span) FailedReason() (bool, []string) {
 }
 
 func (span *Span) IsCanceled() bool {
-	canceled, _ := span.CanceledReason()
-	return canceled
+	return span.Canceled || len(span.CanceledLinks.Order) > 0
 }
 
 func (span *Span) CanceledReason() (bool, []string) {
@@ -459,7 +707,9 @@ func (span *Span) CanceledReason() (bool, []string) {
 		return span.Canceled_, span.CanceledReason_
 	}
 	var reasons []string
-	if span.Canceled {
+	if span.LeftRunning {
+		reasons = append(reasons, "span was left running after the root span completed")
+	} else if span.Canceled {
 		reasons = append(reasons, "span says it is canceled")
 	}
 	for _, canceled := range span.CanceledLinks.Order {
@@ -563,8 +813,29 @@ func (span *Span) IsRunningOrEffectsRunning() bool {
 }
 
 func (span *Span) IsPending() bool {
-	pending, _ := span.PendingReason()
-	return pending
+	// NB: keep this in extremely close alignment with PendingReason, we don't
+	// re-use it so we can minimize allocations
+
+	if span.Final {
+		return span.Pending_
+	}
+	if span.IsRunningOrEffectsRunning() {
+		return false
+	}
+	if len(span.EffectIDs) > 0 {
+		for _, digest := range span.EffectIDs {
+			effectSpans := span.db.EffectSpans[digest]
+			if effectSpans != nil && len(effectSpans.Order) > 0 {
+				return false
+			}
+			if span.db.CompletedEffects[digest] {
+				return false
+			}
+		}
+		// there's an output but no linked spans yet, so we're pending
+		return true
+	}
+	return false
 }
 
 func (span *Span) PendingReason() (bool, []string) {
@@ -604,8 +875,45 @@ func (span *Span) PendingReason() (bool, []string) {
 }
 
 func (span *Span) IsCached() bool {
-	cached, _ := span.CachedReason()
-	return cached
+	// NB: keep this in extremely close alignment with CachedReason, we don't
+	// re-use it so we can minimize allocations
+
+	if span.Final {
+		return span.Cached_
+	}
+	if span.Cached {
+		return true
+	}
+	if span.ChildCount > 0 {
+		return false
+	}
+	if span.HasLogs {
+		return false
+	}
+	var anyCached bool
+	for _, effect := range span.EffectIDs {
+		// first check for spans we've seen for the effect
+		effectSpans := span.db.EffectSpans[effect]
+		if effectSpans != nil && len(effectSpans.Order) > 0 {
+			for _, span := range effectSpans.Order {
+				if span.IsCached() {
+					anyCached = true
+				} else {
+					// if any effects were not cache hits, we're definitely not cached
+					return false
+				}
+			}
+		} else if span.db.CompletedEffects[effect] {
+			// if the effect is completed but we never saw a span for it, that
+			// might mean it was a multiple-layers-deep cache hit. or, some
+			// buildkit bug caused us to never see the span. or, another parallel
+			// client completed it. in all of those cases, we'll at least consider
+			// it cached so it's not stuck 'pending' forever.
+			anyCached = true
+		}
+	}
+	// some effects were not cached
+	return anyCached
 }
 
 func (span *Span) CachedReason() (bool, []string) {
