@@ -10,6 +10,7 @@ import (
 
 type Info struct {
 	BuildkitVersion BuildkitVersion `json:"buildkitVersion"`
+	SystemInfo      SystemInfo      `json:"systemInfo"`
 }
 
 type BuildkitVersion struct {
@@ -18,6 +19,10 @@ type BuildkitVersion struct {
 	Revision string `json:"revision"`
 }
 
+type SystemInfo struct {
+	NumCPU int `json:"numCPU"`
+}
+
 func (c *Client) Info(ctx context.Context) (*Info, error) {
 	res, err := c.ControlClient().Info(ctx, &controlapi.InfoRequest{})
 	if err != nil {
@@ -25,6 +30,7 @@ func (c *Client) Info(ctx context.Context) (*Info, error) {
 	}
 	return &Info{
 		BuildkitVersion: fromAPIBuildkitVersion(res.BuildkitVersion),
+		SystemInfo:      fromAPISystemInfo(res.SystemInfo),
 	}, nil
 }
 
@@ -38,3 +44,12 @@ func fromAPIBuildkitVersion(in *apitypes.BuildkitVersion) BuildkitVersion {
 		Revision: in.Revision,
 	}
 }
+
+func fromAPISystemInfo(in *controlapi.SystemInfo) SystemInfo {
+	if in == nil {
+		return SystemInfo{}
+	}
+	return SystemInfo{
+		NumCPU: int(in.NumCPU),
+	}
+}
